from __future__ import annotations

from typing import Any, Sequence

import manim
import numpy as np
import numpy.typing as npt

__all__ = [
    "Square",
    "Polygon",
    "RegularPolygon",
    "Triangle",
    "Circle",
    "Dot",
    "Line",
]


class Square(manim.Square):
    """
    Constructs a square geometry with the specified side length.

    An alias class for manim.Square.

    Args:
        side_length:
            square's side length
    """

    def __init__(self, side_length: float, **kwargs: dict[str, Any]) -> None:
        self.side_length = side_length
        manim.Square.__init__(self, self.side_length, **kwargs)

    def __repr__(self) -> str:
        return f"Square(side_length={self.side_length}) (alias for manim.Square)"


class Polygon(manim.Polygon):
    """
    Constructs a shape consisting of one close loop of vertices.

    An alias class for manim.Polygon.

    Args:
        *vertices:
            Polygon's vertices points
    """

    def __init__(
        self, *vertices: Sequence[Sequence[float]], **kwargs: dict[str, Any]
    ) -> None:
        self.vertices = vertices
        manim.Polygon.__init__(self, *vertices, **kwargs)

    def __repr__(self) -> str:
        return (
            f"Polygon(vertices={[v for v in self.vertices]}) (alias for manim.Polygon)"
        )


class RegularPolygon(manim.RegularPolygon):
    """
    Constructs a n-sided regular Polygon.

    An alias class for manim.RegularPolygon.

    Args:
        n:
            number of sides of the RegularPolygon
    """

    def __init__(self, n: int, **kwargs: dict[str, Any]) -> None:
        self.n = n
        manim.RegularPolygon.__init__(self, self.n, **kwargs)

    def __repr__(self) -> str:
        return f"RegularPolygon(n={self.n}) (alias for manim.RegularPolygon)"


class Triangle(manim.Triangle):
    """
    Constructs an equilateral Triangle geometry.

    An alias class for manim.Triangle.

    """

    def __init__(self, **kwargs: dict[str, Any]) -> None:
        manim.Triangle.__init__(self, **kwargs)

    def __repr__(self) -> str:
        return "Triangle() (alias for manim.Triangle)"


class Circle(manim.Circle):
    """
    Constructs a circular geometry with specified radius.

    An alias class for manim.Circle.

    Args:
        radius:
            the radius of the circle
    """

    def __init__(self, radius: float, **kwargs: dict[str, Any]) -> None:
        self.radius = radius
        manim.Circle.__init__(self, self.radius, **kwargs)

    def __repr__(self) -> str:
        return f"Circle(radius={self.radius}) (alias for manim.Circle)"


class Dot(manim.Dot):
    """
    Constructs a circle with a very small radius.

    An alias class for manim.Dot.

    """

    def __init__(
        self,
        point: list[float] | npt.NDArray[np.float64],
        radius: float,
        **kwargs: dict[str, Any],
    ) -> None:
        self.radius = radius
        self.point = point
        manim.Dot.__init__(self, point=self.point, radius=self.radius, **kwargs)

    def __repr__(self) -> str:
        return f"Dot(point={self.point}, radius={self.radius}) (alias for manim.Dot)"


class Line(manim.Line):
    """
    Constructs a Line geometry with the specified staring and end points.

    An alias class for manim.Line.

    Args:
        start:
            Line's starting points
        end:
            Line's end points
    """

    def __init__(
        self,
        start: Sequence[float] | npt.NDArray[np.float64],
        end: Sequence[float] | npt.NDArray[np.float64],
        **kwargs: dict[str, Any],
    ):
        self.start = start
        self.end = end
        manim.Line.__init__(self, start=self.start, end=self.end, **kwargs)

    def __repr__(self) -> str:
        return f"Line(start={self.start}, end={self.end}) (alias for manim.Line)"



from __future__ import annotations

import manim

import riemapp as rp


def test_triangle():
    triangle = rp.geometry.Triangle()
    assert triangle.__repr__() == "Triangle() (alias for manim.Triangle)"
    assert isinstance(triangle, (rp.geometry.Triangle, manim.Triangle))


def test_square():
    square = rp.geometry.Square(side_length=2.0)
    assert (
        square.__repr__()
        == f"Square(side_length={square.side_length}) (alias for manim.Square)"
    )
    assert isinstance(square, (rp.geometry.Square, manim.Square))
    assert square.side_length == 2.0


def test_circle():
    circle = rp.geometry.Circle(radius=1.0)
    assert (
        circle.__repr__() == f"Circle(radius={circle.radius}) (alias for manim.Circle)"
    )
    assert isinstance(circle, (rp.geometry.Circle, manim.Circle))
    assert circle.radius == 1.0


def test_line():
    line = rp.geometry.Line(start=[2], end=[4])
    assert (
        line.__repr__()
        == f"Line(start={line.start}, end={line.end}) (alias for manim.Line)"
    )
    assert isinstance(line, (rp.geometry.Line, manim.Line))
    assert line.start == [2]
    assert line.end == [4]


def test_regularpolygon():
    regularpolygon = rp.geometry.RegularPolygon(n=6)
    assert (
        regularpolygon.__repr__()
        == f"RegularPolygon(n={regularpolygon.n}) (alias for manim.RegularPolygon)"
    )
    assert isinstance(
        regularpolygon, (rp.geometry.RegularPolygon, manim.RegularPolygon)
    )
    assert regularpolygon.n == 6


def test_polygon():
    polygon = rp.geometry.Polygon([2, 4, 8], [1, 0, 0], [4, 4, 8])
    assert (
        polygon.__repr__()
        == f"Polygon(vertices={[v for v in polygon.vertices]}) (alias for manim.Polygon)"
    )
    assert isinstance(polygon, (rp.geometry.Polygon, manim.Polygon))
    assert polygon.vertices == ([2, 4, 8], [1, 0, 0], [4, 4, 8])


def test_dot():
    dot = rp.geometry.Dot([0, 0, 0], 0.88)
    assert (
        dot.__repr__()
        == f"Dot(point={dot.point}, radius={dot.radius}) (alias for manim.Dot)"
    )
    assert isinstance(dot, (rp.geometry.Dot, manim.Dot))
    assert dot.point == [0, 0, 0]
    assert dot.radius == 0.88



from __future__ import annotations

import os

import numpy as np
import pytest

import riemapp as rp


def test_complex_map():

    f = rp.geometry.Square(2.0)

    def transform(z):
        return np.e**z

    cm = rp.ComplexMap(f, transform)
    assert isinstance(cm, rp.ComplexMap)
    assert cm.f == f
    assert cm.transformation == transform

    with pytest.raises(ValueError):
        cm.render()

    animation = cm.generate_animation()
    assert isinstance(cm.animate, cm.Animate)
    assert animation == cm.animate

    cm.render()
    assert os.path.exists("./media/videos/1080p60/Animate.mp4")



from __future__ import annotations

from typing import Callable

from manim import ApplyComplexFunction, Create, NumberPlane, Scene

from riemapp.geometry import (
    Circle,
    Dot,
    Line,
    Polygon,
    RegularPolygon,
    Square,
    Triangle,
)

__all__ = ["ComplexMap"]


class ComplexMap:
    """
    Construct and animate a complex mapping.

    Args:
        f:
            A geometry (or `MObject`) created using `riemapp.geometry` or `manim`.
        transformation:
            The complex transformation as a function reference.
    """

    def __init__(
        self,
        f: Square | Polygon | RegularPolygon | Triangle | Circle | Dot | Line,
        transformation: Callable[[float], float],
    ) -> None:
        self.f = f
        self.transformation = transformation

    def __repr__(self) -> str:
        return (
            f"ComplexMap(f={self.f!r}, transformation={self.transformation.__name__})"
        )

    def generate_animation(
        self, *, add_numberplane: bool = False, run_time: float = 1.0
    ) -> ComplexMap.Animate:
        """
        Generates a mathematical animation.

        Args:
            add_numberplane:
                Adds NumberPlane to the scene.
            run_time:
                Run time for creating the provided geometry.

        Returns:
            animate:
                A custom manim Scene object
        """
        self.animate = self.Animate(
            self.f,
            self.transformation,
            add_numberplane=add_numberplane,
            run_time=run_time,
        )

        return self.animate

    def render(self, preview: bool = False) -> None:
        """
        Renders the animation.

        Args:
            preview:
                Automatically opens the generated animation.
        """
        if not hasattr(self, "animate"):
            raise ValueError("generate an animation first")

        self.animate.render(preview=preview)

    class Animate(Scene):
        """
        A placeholder class for manim Scene.

        This class is used to construct and generate manim animations programatically.

        Args:
            f:
                A geometry (or `MObject`) created using `riemapp.geometry` or `manim`.
            transformation:
                The complex transformation as a function reference.
            add_numberplane:
                Adds NumberPlane to the scene.
            run_time:
                Run time for creating the provided geometry.

        """

        def __init__(
            self,
            f: Square | Polygon | RegularPolygon | Triangle | Circle | Dot | Line,
            transformation: Callable[[float], float],
            *,
            add_numberplane: bool = False,
            run_time: float = 1.0,
        ) -> None:
            self.add_numberplane = add_numberplane
            self.run_time = run_time
            self.f = f
            self.transformation = transformation
            super().__init__()

        def __repr__(self) -> str:
            return (
                f"Animate(f={self.f!r}, transformation={self.transformation.__name__})"
            )

        def construct(self) -> None:
            """
            The default manim constructer
            """
            self.add_numberplane and self.add(NumberPlane())
            self.play(Create(self.f, run_time=self.run_time))
            self.play(
                ApplyComplexFunction(self.transformation, self.f),
            )



"""
Copyright (c) 2022 Saransh Chopra. All rights reserved.

riemapp: Manim powered complex mappings.
"""

from __future__ import annotations

import riemapp.geometry  # noqa: F401
from riemapp.core import ComplexMap  # noqa: F401
from riemapp.version import version as __version__  # noqa: F401
